@@ -3,4 +3,4 @@ from .classification_metrics import Precision, PrecisionAtK, RPrecision, Recall,
 from .error_metrics import MAE, MSE, RMSE
 from .fairness_metrics import GiniIndex, DeltaGap, PredictionCoverage, CatalogCoverage
 from .plot_metrics import PopRatioProfileVsRecs, PopRecsCorrelation, LongTailDistr
-from .ranking_metrics import NDCG, NDCGAtK, MRR, MRRAtK, Correlation
+from .ranking_metrics import NDCG, NDCGAtK, MRR, MRRAtK, Correlation, MAP
