@@ -6,5 +6,5 @@ from .content_based_algorithm import *
 from .graph_based_algorithm import *
 from .graphs import *
 from .recsys import ContentBasedRS, GraphBasedRS
-from .partitioning import KFoldPartitioning, HoldOutPartitioning, Split
+from .partitioning import KFoldPartitioning, HoldOutPartitioning, Split, BootstrapPartitioning
 from .methodology import TestRatingsMethodology, TestItemsMethodology, TrainingItemsMethodology, AllItemsMethodology
