@@ -1,5 +1,6 @@
 import abc
 import gc
+import itertools
 from copy import deepcopy
 from typing import Union, Dict, List, Optional
 
@@ -15,7 +16,8 @@ from clayrs.recsys.content_based_algorithm.content_based_algorithm import Conten
 from clayrs.recsys.content_based_algorithm.exceptions import UserSkipAlgFit, NotFittedAlg
 from clayrs.recsys.graph_based_algorithm.graph_based_algorithm import GraphBasedAlgorithm
 from clayrs.recsys.methodology import Methodology
-from clayrs.utils.const import logger, get_progbar
+from clayrs.utils.const import logger
+from clayrs.utils.context_managers import get_iterator_parallel
 
 
 class RecSys(ABC):
@@ -147,30 +149,38 @@ class ContentBasedRS(RecSys):
         """
         return self.__users_directory
 
-    def fit(self):
+    def fit(self, num_cpus: int = 0):
         """
         Method which will fit the algorithm chosen for each user in the train set passed in the constructor
 
         If the algorithm can't be fit for some users, a warning message is printed
         """
+        def compute_single_fit(user_id):
+            user_train = self.train_set.get_user_interactions(user_id)
+            user_alg = deepcopy(self.algorithm)
+
+            try:
+                user_alg.process_rated(user_train, loaded_items_interface)
+                user_alg.fit()
+            except UserSkipAlgFit as e:
+                warning_message = str(e) + f"\nNo algorithm will be fitted for the user {user_id}"
+                logger.warning(warning_message)
+                user_alg = None
+
+            return user_id, user_alg
+
         items_to_load = set(self.train_set.item_id_column)
+        all_users = set(self.train_set.user_id_column)
         loaded_items_interface = self.algorithm._load_available_contents(self.items_directory, items_to_load)
 
-        with get_progbar(set(self.train_set.user_id_column)) as pbar:
+        with get_iterator_parallel(num_cpus,
+                                   compute_single_fit, all_users,
+                                   progress_bar=True, total=len(all_users)) as pbar:
 
             pbar.set_description("Fitting algorithm")
-            for user_id in pbar:
-                user_train = self.train_set.get_user_interactions(user_id)
 
-                try:
-                    user_alg = deepcopy(self.algorithm)
-                    user_alg.process_rated(user_train, loaded_items_interface)
-                    user_alg.fit()
-                    self._user_fit_dic[user_id] = user_alg
-                except UserSkipAlgFit as e:
-                    warning_message = str(e) + f"\nNo algorithm will be fitted for the user {user_id}"
-                    logger.warning(warning_message)
-                    self._user_fit_dic[user_id] = None
+            for user_id, fitted_user_alg in pbar:
+                self._user_fit_dic[user_id] = fitted_user_alg
 
         # we force the garbage collector after freeing loaded items
         del loaded_items_interface
@@ -179,7 +189,9 @@ class ContentBasedRS(RecSys):
         return self
 
     def rank(self, test_set: Ratings, n_recs: int = None, user_id_list: List = None,
-             methodology: Union[Methodology, None] = TestRatingsMethodology()) -> Rank:
+             methodology: Union[Methodology, None] = TestRatingsMethodology(),
+             num_cpus: int = 0) -> Rank:
+
         """
         Method used to calculate ranking for all users in test set or all users in `user_id_list` parameter.
         You must first call the `fit()` method before you can compute the ranking.
@@ -200,6 +212,8 @@ class ContentBasedRS(RecSys):
                 for all users of the `test_set`
             methodology: `Methodology` object which governs the candidate item selection. Default is
                 `TestRatingsMethodology`
+            num_cpus: number of processors that must be reserved for the method. Default is 0, meaning that
+                the number of cpus will be automatically detected.
 
         Raises:
             NotFittedAlg: Exception raised when this method is called without first calling the `fit` method
@@ -208,6 +222,24 @@ class ContentBasedRS(RecSys):
             Rank object containing recommendation lists for all users of the test set or for all users in `user_id_list`
 
         """
+        def compute_single_rank(user_id):
+            user_id = str(user_id)
+            user_train = self.train_set.get_user_interactions(user_id)
+
+            filter_list = None
+            if methodology is not None:
+                filter_list = set(methodology.filter_single(user_id, self.train_set, test_set))
+
+            user_fitted_alg = self._user_fit_dic.get(user_id)
+            if user_fitted_alg is not None:
+                user_rank = user_fitted_alg.rank(user_train, loaded_items_interface,
+                                                 n_recs, filter_list=filter_list)
+            else:
+                user_rank = []
+                logger.warning(f"No algorithm fitted for user {user_id}! It will be skipped")
+
+            return user_id, user_rank
+
         if len(self._user_fit_dic) == 0:
             raise NotFittedAlg("Algorithm not fit! You must call the fit() method first, or fit_rank().")
 
@@ -221,28 +253,17 @@ class ContentBasedRS(RecSys):
 
         logger.info("Don't worry if it looks stuck at first")
         logger.info("First iterations will stabilize the estimated remaining time")
-        with get_progbar(all_users) as pbar:
+
+        with get_iterator_parallel(num_cpus,
+                                   compute_single_rank, all_users,
+                                   progress_bar=True, total=len(all_users)) as pbar:
 
             pbar.set_description(f"Loading first items from memory...")
-            for user_id in pbar:
-                user_id = str(user_id)
-                user_train = self.train_set.get_user_interactions(user_id)
-
-                filter_list = None
-                if methodology is not None:
-                    filter_list = set(methodology.filter_single(user_id, self.train_set, test_set))
-
-                user_fitted_alg = self._user_fit_dic.get(user_id)
-                if user_fitted_alg is not None:
-                    user_rank = user_fitted_alg.rank(user_train, loaded_items_interface,
-                                                     n_recs, filter_list=filter_list)
-                else:
-                    user_rank = []
-                    logger.warning(f"No algorithm fitted for user {user_id}! It will be skipped")
-
-                rank.extend(user_rank)
-                pbar.set_description(f"Computing rank for {user_id}")
+            for user_id, user_rank in pbar:
+                pbar.set_description(f"Computing rank for user {user_id}")
+                rank.append(user_rank)
 
+        rank = itertools.chain.from_iterable(rank)
         rank = Rank.from_list(rank)
 
         # we force the garbage collector after freeing loaded items
@@ -254,7 +275,8 @@ class ContentBasedRS(RecSys):
         return rank
 
     def predict(self, test_set: Ratings, user_id_list: List = None,
-                methodology: Union[Methodology, None] = TestRatingsMethodology()) -> Prediction:
+                methodology: Union[Methodology, None] = TestRatingsMethodology(),
+                num_cpus: int = 0) -> Prediction:
         """
         Method used to calculate score predictions for all users in test set or all users in `user_id_list` parameter.
         You must first call the `fit()` method before you can compute score predictions.
@@ -273,6 +295,8 @@ class ContentBasedRS(RecSys):
                 will be computed for all users of the `test_set`
             methodology: `Methodology` object which governs the candidate item selection. Default is
                 `TestRatingsMethodology`
+            num_cpus: number of processors that must be reserved for the method. Default is 0, meaning that
+                the number of cpus will be automatically detected.
 
         Raises:
             NotFittedAlg: Exception raised when this method is called without first calling the `fit` method
@@ -282,6 +306,23 @@ class ContentBasedRS(RecSys):
                 `user_id_list`
 
         """
+        def compute_single_predict(user_id):
+            user_id = str(user_id)
+            user_train = self.train_set.get_user_interactions(user_id)
+
+            filter_list = None
+            if methodology is not None:
+                filter_list = set(methodology.filter_single(user_id, self.train_set, test_set))
+
+            user_fitted_alg = self._user_fit_dic.get(user_id)
+            if user_fitted_alg is not None:
+                user_pred = user_fitted_alg.predict(user_train, loaded_items_interface, filter_list=filter_list)
+            else:
+                user_pred = []
+                logger.warning(f"No algorithm fitted for user {user_id}! It will be skipped")
+
+            return user_id, user_pred
+
         if len(self._user_fit_dic) == 0:
             raise NotFittedAlg("Algorithm not fit! You must call the fit() method first, or fit_rank().")
 
@@ -295,29 +336,17 @@ class ContentBasedRS(RecSys):
 
         logger.info("Don't worry if it looks stuck at first")
         logger.info("First iterations will stabilize the estimated remaining time")
-        with get_progbar(all_users) as pbar:
+
+        with get_iterator_parallel(num_cpus,
+                                   compute_single_predict, all_users,
+                                   progress_bar=True, total=len(all_users)) as pbar:
 
             pbar.set_description(f"Loading first items from memory...")
-            for user_id in pbar:
-                user_id = str(user_id)
-
-                user_train = self.train_set.get_user_interactions(user_id)
-
-                filter_list = None
-                if methodology is not None:
-                    filter_list = set(methodology.filter_single(user_id, self.train_set, test_set))
-
-                user_fitted_alg = self._user_fit_dic.get(user_id)
-                if user_fitted_alg is not None:
-                    user_pred = user_fitted_alg.predict(user_train, loaded_items_interface,
-                                                        filter_list=filter_list)
-                else:
-                    user_pred = []
-                    logger.warning(f"No algorithm fitted for user {user_id}! It will be skipped")
-
-                pred.extend(user_pred)
-                pbar.set_description(f"Computing predictions for user {user_id}")
+            for user_id, user_pred in pbar:
+                pbar.set_description(f"Computing score prediction for user {user_id}")
+                pred.append(user_pred)
 
+        pred = itertools.chain.from_iterable(pred)
         pred = Prediction.from_list(pred)
 
         # we force the garbage collector after freeing loaded items
@@ -330,7 +359,7 @@ class ContentBasedRS(RecSys):
 
     def fit_predict(self, test_set: Ratings, user_id_list: List = None,
                     methodology: Union[Methodology, None] = TestRatingsMethodology(),
-                    save_fit: bool = False) -> Prediction:
+                    save_fit: bool = False, num_cpus: int = 0) -> Prediction:
         """
         Method used to both fit and calculate score prediction for all users in test set or all users in `user_id_list`
         parameter.
@@ -358,10 +387,39 @@ class ContentBasedRS(RecSys):
                 `TestRatingsMethodology`
             save_fit: Boolean value which let you choose if the Recommender System should remain fit even after the
                 complete execution of this method. Default is False
+            num_cpus: number of processors that must be reserved for the method. Default is 0, meaning that
+                the number of cpus will be automatically detected.
 
         Returns:
             Rank object containing recommendation lists for all users of the test set or for all users in `user_id_list`
         """
+        def compute_single_fit_predict(user_id):
+            user_train = self.train_set.get_user_interactions(user_id)
+
+            alg = self.algorithm
+            if save_fit:
+                alg = deepcopy(alg)
+                self._user_fit_dic[user_id] = alg
+
+            try:
+                alg.process_rated(user_train, loaded_items_interface)
+                alg.fit()
+
+            except UserSkipAlgFit as e:
+                warning_message = str(e) + f"\nThe algorithm can't be fitted for the user {user_id}"
+                logger.warning(warning_message)
+                if save_fit:
+                    self._user_fit_dic[user_id] = None
+                return
+
+            filter_list = None
+            if methodology is not None:
+                filter_list = set(methodology.filter_single(user_id, self.train_set, test_set))
+
+            user_pred = alg.predict(user_train, loaded_items_interface, filter_list=filter_list)
+
+            return user_id, user_pred
+
         all_users = set(test_set.user_id_column)
         if user_id_list is not None:
             all_users = set(user_id_list)
@@ -372,43 +430,17 @@ class ContentBasedRS(RecSys):
 
         logger.info("Don't worry if it looks stuck at first")
         logger.info("First iterations will stabilize the estimated remaining time")
-        with get_progbar(all_users) as pbar:
+
+        with get_iterator_parallel(num_cpus,
+                                   compute_single_fit_predict, all_users,
+                                   progress_bar=True, total=len(all_users)) as pbar:
 
             pbar.set_description(f"Loading first items from memory...")
-            for user_id in pbar:
-                user_id = str(user_id)
-
-                user_train = self.train_set.get_user_interactions(user_id)
-
-                try:
-                    if save_fit:
-                        user_alg = deepcopy(self.algorithm)
-                        self._user_fit_dic[user_id] = user_alg
-                        alg = user_alg
-                    else:
-                        alg = self.algorithm
-
-                    alg.process_rated(user_train, loaded_items_interface)
-
-                    alg.fit()
-
-                except UserSkipAlgFit as e:
-                    warning_message = str(e) + f"\nThe algorithm can't be fitted for the user {user_id}"
-                    logger.warning(warning_message)
-                    if save_fit:
-                        self._user_fit_dic[user_id] = None
-                    continue
-
-                filter_list = None
-                if methodology is not None:
-                    filter_list = set(methodology.filter_single(user_id, self.train_set, test_set))
-
-                user_pred = alg.predict(user_train, loaded_items_interface,
-                                        filter_list=filter_list)
-
-                pred.extend(user_pred)
-                pbar.set_description(f"Computing fit_rank for user {user_id}")
+            for user_id, user_pred in pbar:
+                pbar.set_description(f"Computing fit_predict for user {user_id}")
+                pred.append(user_pred)
 
+        pred = itertools.chain.from_iterable(pred)
         pred = Prediction.from_list(pred)
 
         # we force the garbage collector after freeing loaded items
@@ -421,7 +453,7 @@ class ContentBasedRS(RecSys):
 
     def fit_rank(self, test_set: Ratings, n_recs: int = None, user_id_list: List = None,
                  methodology: Union[Methodology, None] = TestRatingsMethodology(),
-                 save_fit: bool = False) -> Rank:
+                 save_fit: bool = False, num_cpus: int = 0) -> Rank:
         """
         Method used to both fit and calculate ranking for all users in test set or all users in `user_id_list`
         parameter.
@@ -451,6 +483,8 @@ class ContentBasedRS(RecSys):
                 `TestRatingsMethodology`
             save_fit: Boolean value which let you choose if the Recommender System should remain fit even after the
                 complete execution of this method. Default is False
+            num_cpus: number of processors that must be reserved for the method. Default is 0, meaning that
+                the number of cpus will be automatically detected.
 
         Raises:
             NotFittedAlg: Exception raised when this method is called without first calling the `fit` method
@@ -458,6 +492,34 @@ class ContentBasedRS(RecSys):
         Returns:
             Rank object containing recommendation lists for all users of the test set or for all users in `user_id_list`
         """
+        def compute_single_fit_rank(user_id):
+            user_train = self.train_set.get_user_interactions(user_id)
+
+            alg = self.algorithm
+            if save_fit:
+                alg = deepcopy(alg)
+                self._user_fit_dic[user_id] = alg
+
+            try:
+                alg.process_rated(user_train, loaded_items_interface)
+                alg.fit()
+
+            except UserSkipAlgFit as e:
+                warning_message = str(e) + f"\nThe algorithm can't be fitted for the user {user_id}"
+                logger.warning(warning_message)
+                if save_fit:
+                    self._user_fit_dic[user_id] = None
+                return
+
+            filter_list = None
+            if methodology is not None:
+                filter_list = set(methodology.filter_single(user_id, self.train_set, test_set))
+
+            user_rank = alg.rank(user_train, loaded_items_interface,
+                                 n_recs, filter_list=filter_list)
+
+            return user_id, user_rank
+
         all_users = set(test_set.user_id_column)
         if user_id_list is not None:
             all_users = set(user_id_list)
@@ -468,43 +530,17 @@ class ContentBasedRS(RecSys):
 
         logger.info("Don't worry if it looks stuck at first")
         logger.info("First iterations will stabilize the estimated remaining time")
-        with get_progbar(all_users) as pbar:
+
+        with get_iterator_parallel(num_cpus,
+                                   compute_single_fit_rank, all_users,
+                                   progress_bar=True, total=len(all_users)) as pbar:
 
             pbar.set_description(f"Loading first items from memory...")
-            for user_id in pbar:
-                user_id = str(user_id)
-
-                user_train = self.train_set.get_user_interactions(user_id)
-
-                try:
-                    if save_fit:
-                        user_alg = deepcopy(self.algorithm)
-                        self._user_fit_dic[user_id] = user_alg
-                        alg = user_alg
-                    else:
-                        alg = self.algorithm
-
-                    alg.process_rated(user_train, loaded_items_interface)
-
-                    alg.fit()
-
-                except UserSkipAlgFit as e:
-                    warning_message = str(e) + f"\nThe algorithm can't be fitted for the user {user_id}"
-                    logger.warning(warning_message)
-                    if save_fit:
-                        self._user_fit_dic[user_id] = None
-                    continue
-
-                filter_list = None
-                if methodology is not None:
-                    filter_list = set(methodology.filter_single(user_id, self.train_set, test_set))
-
-                user_rank = alg.rank(user_train, loaded_items_interface,
-                                     n_recs, filter_list=filter_list)
-
-                rank.extend(user_rank)
+            for user_id, user_rank in pbar:
                 pbar.set_description(f"Computing fit_rank for user {user_id}")
+                rank.append(user_rank)
 
+        rank = itertools.chain.from_iterable(rank)
         rank = Rank.from_list(rank)
 
         # we force the garbage collector after freeing loaded items
@@ -604,7 +640,8 @@ class GraphBasedRS(RecSys):
         return alg
 
     def predict(self, test_set: Ratings, user_id_list: List = None,
-                methodology: Union[Methodology, None] = TestRatingsMethodology()) -> Prediction:
+                methodology: Union[Methodology, None] = TestRatingsMethodology(),
+                num_cpus: int = 0) -> Prediction:
         """
         Method used to calculate score predictions for all users in test set or all users in `user_id_list` parameter.
 
@@ -622,6 +659,8 @@ class GraphBasedRS(RecSys):
                 will be computed for all users of the `test_set`
             methodology: `Methodology` object which governs the candidate item selection. Default is
                 `TestRatingsMethodology`
+            num_cpus: number of processors that must be reserved for the method. Default is 0, meaning that
+                the number of cpus will be automatically detected.
 
         Returns:
             Prediction object containing score prediction lists for all users of the test set or for all users in
@@ -632,12 +671,7 @@ class GraphBasedRS(RecSys):
         if user_id_list is not None:
             all_users = set(user_id_list)
 
-        filter_dict: Union[Dict, None] = None
-        if methodology is not None:
-            train_set = self.graph.to_ratings()
-            filter_dict = methodology.filter_all(train_set, test_set, result_as_iter_dict=True)
-
-        total_predict_list = self.algorithm.predict(all_users, self.graph, filter_dict)
+        total_predict_list = self.algorithm.predict(all_users, self.graph, test_set, methodology, num_cpus)
 
         total_predict = Prediction.from_list(total_predict_list)
 
@@ -646,7 +680,8 @@ class GraphBasedRS(RecSys):
         return total_predict
 
     def rank(self, test_set: Ratings, n_recs: int = None, user_id_list: List = None,
-             methodology: Union[Methodology, None] = TestRatingsMethodology()) -> Rank:
+             methodology: Union[Methodology, None] = TestRatingsMethodology(),
+             num_cpus: int = 0) -> Rank:
         """
         Method used to calculate ranking for all users in test set or all users in `user_id_list` parameter.
         You must first call the `fit()` method before you can compute the ranking.
@@ -667,6 +702,8 @@ class GraphBasedRS(RecSys):
                 for all users of the `test_set`
             methodology: `Methodology` object which governs the candidate item selection. Default is
                 `TestRatingsMethodology`
+            num_cpus: number of processors that must be reserved for the method. Default is 0, meaning that
+                the number of cpus will be automatically detected.
 
         Raises:
             NotFittedAlg: Exception raised when this method is called without first calling the `fit` method
@@ -679,12 +716,7 @@ class GraphBasedRS(RecSys):
         if user_id_list is not None:
             all_users = set(user_id_list)
 
-        filter_dict: Union[Dict, None] = None
-        if methodology is not None:
-            train_set = self.graph.to_ratings()
-            filter_dict = methodology.filter_all(train_set, test_set, result_as_iter_dict=True)
-
-        total_rank_list = self.algorithm.rank(all_users, self.graph, n_recs, filter_dict)
+        total_rank_list = self.algorithm.rank(all_users, self.graph, test_set, n_recs, methodology, num_cpus)
 
         total_rank = Rank.from_list(total_rank_list)
 
